"""Nuke internal menu.py which will register all UI elements."""

# Import third-party modules
# pylint: disable=import-error
import nuke
import nukescripts

# Import local modules
from smartLink import helper
from smartLink import knob_code
from smartLink import nodes
from smartLink import osl as smartlink
from smartLink import overwrites
from smartLink import shortcuteditor


# All icons automatically being scanned from icons directory.
ICONS = helper.load_icons()


def add_command(menu, label, callable_, shortcut="", icon=None, context=2):
    """Add command using shortcut context when applicable.

    The 'shortcutContext' has been implemented in Nuke9. Implement it only when
    we are in Nuke9 and above.

    Args:
        menu (nuke.MenuItem): Menu item to add command to.
        label (str): Label of command as it will appear in ui.
        callable_ (function): Function to execute when being invoked.
        shortcut (str): Shortcut combination to use to invoke command.
        icon (str): Name of icon to use. Must be in NUKE_PATH.
        context (int): Index of shortcut context.
            0=Window, 1=Application, 2=DAG

    """
    if nuke.NUKE_VERSION_MAJOR >= 9:
        menu.addCommand(label, callable_, icon=icon, shortcut=shortcut,
                        shortcutContext=context)
    else:
        menu.addCommand(label, callable_, icon=icon, shortcut=shortcut)


def overwrite_commands():
    """Overwrite Nuke internal commands to autoconnect representation nodes.

    By default, Nuke doesn't offer the functionality to connect a
    representation node (i.e. a PostageStamp node) to it's source when we copy
    and paste it. Here we overwrite the Nuke internal 'paste' and 'duplicate'
    commands to offer exactly that. Additionally, we will update source id knob
    values of our pasted nodes in order to ensure that we have unique source
    id knobs.

    This is save to overwrite, but in case you don't want that behaviour, feel
    free to remove or comment this overwrite out.

    """
    edit = nuke.menu("Nuke").findItem("Edit")

    add_command(edit, "Paste", overwrites.paste, "^v")
    add_command(edit, "Duplicate", overwrites.duplicate, "#c")


def register_menu():
    """Register menu and commands."""
    menu = nuke.menu("Nuke").addMenu("cragl/smartLink", icon=ICONS["logo"])

    add_command(menu, "SmartLinker", lambda: smartlink.show_smartlinker(
        floating=True))

    add_command(menu, "Backdrop Manager", smartlink.show_backdropmanager)

    navigation = menu.addMenu("Navigation")

    add_command(navigation, "Jump to source node",
                knob_code.jump_to_source_node, "Ctrl+Up")

    add_command(navigation, "Previous Link",
                lambda: knob_code.cycle_node("previous"), "Ctrl+Left")

    add_command(navigation, "Next Link",
                lambda: knob_code.cycle_node("next"), "Ctrl+Right")

    add_command(menu, "Create Links", nodes.create_links)
    add_command(menu, "Add selection to favorites", helper.add_to_favorites)
    add_command(menu, "Shortcut Editor", shortcuteditor.show)

    menu.addSeparator()
    add_command(menu, "&About", smartlink.show_about_window)


def register_pane():
    """Register smartLinker to pane."""
    nukescripts.panels.registerWidgetAsPanel(
        "smartlink.show_smartlinker", "smartLinker",
        "com.cragl.smartLink")


def init():
    """Init smartLinker ui elements and callbacks.

    This will execute the following steps:
        - Add smartLinker ui menu in 'cragl -> smartLink'.
        - Register the smartLinker dockable widget in the pane menu.
        - Overwrite the paste and duplicate command to auto connect link nodes.
        - auto update the dockable smartLinker window on node creation.
        - Add custom callback for dragging a link button into the DAG.
        - Set ui shortcuts from settings.

    """
    register_menu()
    register_pane()
    overwrite_commands()
    overwrites.auto_update_dockable_smartlinker()
    nukescripts.addDropDataCallback(nodes.drop_link)
    shortcuteditor.init_shortcuts()

init()
